#!/usr/bin/python
#
# Copyright (C) 2014, Martin Owens <doctormo@gmail.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# Funding contributon from David Prommer
#
"""
A script for generating calendars via SVG documents.
"""

import sys
import os
import itertools
import calendar

from datetime import date

__version__ = 0.9

opts = {}
opt_location = '.'
opt_year = date.today().year

for arg in sys.argv[1:]:
    if arg[0] != '-':
        try:
            opt_year = int(arg)
        except:
            if os.path.isdir(arg):
                opt_location = arg
            else:
                print " ! Unknown argument: '%s' ignored" % arg
                sys.exit(1)
    elif '=' in arg:
        opts.update(dict((arg.strip('-').split('='),)))
    else:
        opts[arg.strip('-')] = True

templates = os.path.join(opt_location, 'templates')
caches = './output/'

if not os.path.isdir(templates):
    print " ! Can't find templates directory '%s'" % templates
    sys.exit(2)
t_names = [ t.rsplit('.',1)[0] for t in os.listdir(templates) if t[-3:] == 'svg' ]

def grouper(gn, iterable, fillvalue=None):
    "grouper(3, 'ABCDEFG', 'x') --> ABC DEF Gxx"
    args = [iter(iterable)] * gn
    return itertools.izip_longest(fillvalue=fillvalue, *args)

def log(msg):
    if opts.get('v', False):
        print msg

log("Caendar Generator %s" % __version__)

def output(name, datum, prefix="out"):
    """Generate output svg from a template name"""
    out = cached_template(name+'.svg', templates)
    for key in datum.keys():
        pkey = isinstance(key, str) and '$'+key or str(key)
        if pkey not in out:
            print "Can't find '%s' in template." % pkey
        out = out.replace(pkey, str(datum[key]))

    filename = os.path.join(caches, '%s-%s.svg' % (prefix, name))
    log(" < Creating svg: %s" % filename)
    with open(filename, 'w') as fhl:
        fhl.write(out)

TEMPLATES = {}
def cached_template(name, loc):
    """Load a template and cache it for future use"""
    global TEMPLATES
    if name not in TEMPLATES:
        filename = os.path.join(loc, name)
        log(" > Reading template: %s" % filename)
        with open(filename, 'r') as fhl:
            TEMPLATES[name] = ''.join(fhl.readlines())
    return TEMPLATES[name]

def print_month(month, year):
    cal = calendar.Calendar(0)
    secs = [''] * 3 + [ str(i or '') for i in cal.itermonthdays(year, int(month)) ]

    # Truncate any blank weeks
    while not any(secs[:7]):
        secs = secs[7:]

    if len(secs) == 31:
        secs = (['']*7) + secs

    datum = {'year': year, 'month': calendar.month_name[int(month)], 'mon': int(month) }
    dates = cached_template('dates.lst', opt_location).split('\n')

    things = {}
    for date in dates:
        if not ':' in date:
            if date:
                print " ! Incorrect date: %s" % date
            continue
        (monthday, items) = date.split(':')
        (dmonth, dday) = monthday.split('-')
        if int(month) == int(dmonth):
            things[str(int(dday))] = items.split(',')
        
    # Pad out the sections in case we don't have enough.
    secs += [''] * (38 - len(secs))

    if len(secs) > 38:
        print " ! Too many days in month %s!" % str(month)
        return

    # Now populate the sections.
    for x, sec in enumerate(secs[:38], 1):
        datum['%s<' % str(x)] = sec + "<"
        if sec == '':
            bn = u'id="box%s"' % str(x)
            datum[bn] = bn+" style='opacity: 0'"
        if things.has_key(sec):
            for y, item in enumerate(things[sec], 1):
                datum['%sc%s<' % (str(x), str(y))] = item+'<'
        # Clear any extra elements
        for y in range(1, 4):
            key = '%sc%s<' % (str(x), str(y))
            if key not in datum:
                datum[key] = '<'

    for t_name in t_names:
        output(t_name, datum, "%s-%s" % (year, month))


# Make sure we have a place to put them
try:
    os.makedirs(caches)
except OSError:
    pass
# Now generate all our months
for m in range(1,13):
    m = str(m)
    if len(m) == 1:
      m = '0'+m
    try:
        print_month(m, opt_year)
    except Exception, msg:
        print " ! Error in month %s (%s)" % (m, str(msg))
        raise

